package com.dzeio.openhealth.core

import android.app.Dialog
import android.os.Bundle
import android.view.LayoutInflater
import androidx.appcompat.app.AlertDialog
import androidx.fragment.app.DialogFragment
import androidx.lifecycle.ViewModelProvider
import androidx.viewbinding.ViewBinding
import com.google.android.material.dialog.MaterialAlertDialogBuilder

abstract class BaseDialog<VM : BaseViewModel, VB : ViewBinding>(private val viewModelClass: Class<VM>) : DialogFragment() {
    val viewModel by lazy {
        ViewModelProvider(this)[viewModelClass]
    }

    private var _binding: VB? = null
    val binding get() = _binding!!

    /**
     * Setup everything!
     */
    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
        return activity?.let { act ->
            val builder = MaterialAlertDialogBuilder(requireContext())

            _binding = bindingInflater(act.layoutInflater)

            builder.setView(binding.root)

            onBuilderInit(builder)

            val dialog = builder.create()

            onDialogInit(dialog)

            onCreated()

            dialog
        } ?: throw IllegalStateException("Activity cannot be null")
    }

    open fun onBuilderInit(builder: MaterialAlertDialogBuilder): Unit {}

    open fun onDialogInit(dialog: AlertDialog): Unit {}

    open fun onCreated(): Unit {}

    /**
     * Function to inflate the Fragment Bindings
     */
    abstract val bindingInflater: (LayoutInflater) -> VB

    /**
     * Destroy binding
     */
    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}